@@ -9,13 +9,19 @@ use crate::backend::{Backend, BackendApi, Querier, Storage};
 use crate::checksum::Checksum;
 use crate::compatibility::check_wasm;
 use crate::errors::{VmError, VmResult};
+use crate::features::required_features_from_module;
 use crate::instance::{Instance, InstanceOptions};
 use crate::modules::{FileSystemCache, InMemoryCache, PinnedMemoryCache};
 use crate::size::Size;
 use crate::static_analysis::{deserialize_wasm, has_ibc_entry_points};
 use crate::wasm_backend::{compile, make_runtime_store};
 
+const STATE_DIR: &str = "state";
+// Things related to the state of the blockchain.
 const WASM_DIR: &str = "wasm";
+
+const CACHE_DIR: &str = "cache";
+// Cacheable things.
 const MODULES_DIR: &str = "modules";
 
 #[derive(Debug, Default, Clone, Copy)]
@@ -67,8 +73,10 @@ pub struct Cache<A: BackendApi, S: Storage, Q: Querier> {
     type_querier: PhantomData<Q>,
 }
 
+#[derive(PartialEq, Debug)]
 pub struct AnalysisReport {
     pub has_ibc_entry_points: bool,
+    pub required_features: HashSet<String>,
 }
 
 impl<A, S, Q> Cache<A, S, Q>
@@ -91,11 +99,24 @@ where
             memory_cache_size,
             instance_memory_limit,
         } = options;
-        let wasm_path = base_dir.join(WASM_DIR);
-        create_dir_all(&wasm_path)
-            .map_err(|e| VmError::cache_err(format!("Error creating Wasm dir for cache: {}", e)))?;
 
-        let fs_cache = FileSystemCache::new(base_dir.join(MODULES_DIR))
+        let state_path = base_dir.join(STATE_DIR);
+        let cache_path = base_dir.join(CACHE_DIR);
+
+        let wasm_path = state_path.join(WASM_DIR);
+
+        // Ensure all the needed directories exist on disk.
+        for path in [&state_path, &cache_path, &wasm_path].iter() {
+            create_dir_all(path).map_err(|e| {
+                VmError::cache_err(format!(
+                    "Error creating directory {}: {}",
+                    path.display(),
+                    e
+                ))
+            })?;
+        }
+
+        let fs_cache = FileSystemCache::new(cache_path.join(MODULES_DIR))
             .map_err(|e| VmError::cache_err(format!("Error file system cache: {}", e)))?;
         Ok(Cache {
             supported_features,
@@ -167,6 +188,7 @@ where
         let module = deserialize_wasm(&wasm)?;
         Ok(AnalysisReport {
             has_ibc_entry_points: has_ibc_entry_points(&module),
+            required_features: required_features_from_module(&module),
         })
     }
 
@@ -192,8 +214,9 @@ where
 
         // Try to get module from file system cache
         let store = make_runtime_store(Some(cache.instance_memory_limit));
-        if let Some((module, module_size)) = cache.fs_cache.load(checksum, &store)? {
+        if let Some(module) = cache.fs_cache.load(checksum, &store)? {
             cache.stats.hits_fs_cache += 1;
+            let module_size = loupe::size_of_val(&module);
             return cache
                 .pinned_memory_cache
                 .store(checksum, module, module_size);
@@ -203,7 +226,8 @@ where
         let code = self.load_wasm_with_path(&cache.wasm_path, checksum)?;
         let module = compile(&code, Some(cache.instance_memory_limit))?;
         // Store into the fs cache too
-        let module_size = cache.fs_cache.store(checksum, &module)?;
+        cache.fs_cache.store(checksum, &module)?;
+        let module_size = loupe::size_of_val(&module);
         cache
             .pinned_memory_cache
             .store(checksum, module, module_size)
@@ -252,10 +276,11 @@ where
 
         // Get module from file system cache
         let store = make_runtime_store(Some(cache.instance_memory_limit));
-        if let Some((module, module_size)) = cache.fs_cache.load(checksum, &store)? {
+        if let Some(module) = cache.fs_cache.load(checksum, &store)? {
             cache.stats.hits_fs_cache += 1;
             let instance =
                 Instance::from_module(&module, backend, options.gas_limit, options.print_debug)?;
+            let module_size = loupe::size_of_val(&module);
             cache.memory_cache.store(checksum, module, module_size)?;
             return Ok(instance);
         }
@@ -270,7 +295,8 @@ where
         let module = compile(&wasm, Some(cache.instance_memory_limit))?;
         let instance =
             Instance::from_module(&module, backend, options.gas_limit, options.print_debug)?;
-        let module_size = cache.fs_cache.store(checksum, &module)?;
+        cache.fs_cache.store(checksum, &module)?;
+        let module_size = loupe::size_of_val(&module);
         cache.memory_cache.store(checksum, module, module_size)?;
         Ok(instance)
     }
@@ -295,7 +321,7 @@ where
 /// save stores the wasm code in the given directory and returns an ID for lookup.
 /// It will create the directory if it doesn't exist.
 /// Saving the same byte code multiple times is allowed.
-fn save_wasm_to_disk<P: Into<PathBuf>>(dir: P, wasm: &[u8]) -> VmResult<Checksum> {
+fn save_wasm_to_disk(dir: impl Into<PathBuf>, wasm: &[u8]) -> VmResult<Checksum> {
     // calculate filename
     let checksum = Checksum::generate(wasm);
     let filename = checksum.to_hex();
@@ -315,7 +341,7 @@ fn save_wasm_to_disk<P: Into<PathBuf>>(dir: P, wasm: &[u8]) -> VmResult<Checksum
     Ok(checksum)
 }
 
-fn load_wasm_from_disk<P: Into<PathBuf>>(dir: P, checksum: &Checksum) -> VmResult<Vec<u8>> {
+fn load_wasm_from_disk(dir: impl Into<PathBuf>, checksum: &Checksum) -> VmResult<Vec<u8>> {
     // this requires the directory and file to exist
     let path = dir.into().join(checksum.to_hex());
     let mut file = File::open(path)
@@ -337,6 +363,7 @@ mod tests {
     use cosmwasm_std::{coins, Empty};
     use std::fs::OpenOptions;
     use std::io::Write;
+    use std::iter::FromIterator;
     use tempfile::TempDir;
 
     const TESTING_GAS_LIMIT: u64 = 4_000_000;
@@ -348,6 +375,7 @@ mod tests {
     const TESTING_MEMORY_CACHE_SIZE: Size = Size::mebi(200);
 
     static CONTRACT: &[u8] = include_bytes!("../testdata/hackatom.wasm");
+    static IBC_CONTRACT: &[u8] = include_bytes!("../testdata/ibc_reflect.wasm");
 
     fn default_features() -> HashSet<String> {
         features_from_csv("staking")
@@ -362,6 +390,15 @@ mod tests {
         }
     }
 
+    fn make_stargate_testing_options() -> CacheOptions {
+        CacheOptions {
+            base_dir: TempDir::new().unwrap().into_path(),
+            supported_features: features_from_csv("staking,stargate"),
+            memory_cache_size: TESTING_MEMORY_CACHE_SIZE,
+            instance_memory_limit: TESTING_MEMORY_LIMIT,
+        }
+    }
+
     #[test]
     fn save_wasm_works() {
         let cache: Cache<MockApi, MockStorage, MockQuerier> =
@@ -494,7 +531,11 @@ mod tests {
         let checksum = cache.save_wasm(CONTRACT).unwrap();
 
         // Corrupt cache file
-        let filepath = tmp_dir.path().join(WASM_DIR).join(&checksum.to_hex());
+        let filepath = tmp_dir
+            .path()
+            .join(STATE_DIR)
+            .join(WASM_DIR)
+            .join(&checksum.to_hex());
         let mut file = OpenOptions::new().write(true).open(filepath).unwrap();
         file.write_all(b"broken data").unwrap();
 
@@ -921,6 +962,35 @@ mod tests {
         assert_eq!(code, loaded);
     }
 
+    #[test]
+    fn analyze_works() {
+        let cache: Cache<MockApi, MockStorage, MockQuerier> =
+            unsafe { Cache::new(make_stargate_testing_options()).unwrap() };
+
+        let checksum1 = cache.save_wasm(CONTRACT).unwrap();
+        let report1 = cache.analyze(&checksum1).unwrap();
+        assert_eq!(
+            report1,
+            AnalysisReport {
+                has_ibc_entry_points: false,
+                required_features: HashSet::new(),
+            }
+        );
+
+        let checksum2 = cache.save_wasm(IBC_CONTRACT).unwrap();
+        let report2 = cache.analyze(&checksum2).unwrap();
+        assert_eq!(
+            report2,
+            AnalysisReport {
+                has_ibc_entry_points: true,
+                required_features: HashSet::from_iter(vec![
+                    "staking".to_string(),
+                    "stargate".to_string()
+                ]),
+            }
+        );
+    }
+
     #[test]
     fn pin_unpin_works() {
         let cache = unsafe { Cache::new(make_testing_options()).unwrap() };
